import React from 'react';
import { Editor, Node, Operation } from 'slate';
/**
 * Associate the context change listener with the editor.
 */
export declare const EDITOR_TO_CONTEXT_LISTENER: WeakMap<Editor, (children: Node[], operations: Operation[]) => void>;
/**
 * A React context for sharing the `Editor` class, in a way that re-renders the
 * context whenever changes occur.
 */
export declare const SlateContext: React.Context<[Editor] | null>;
/**
 * A wrapper around the provider to handle `onChange` events, because the editor
 * is a mutable singleton so it won't ever register as "changed" otherwise.
 */
export declare const Slate: (props: {
    editor: Editor;
    children: JSX.Element;
    defaultValue?: Node[] | undefined;
    onChange?: ((children: Node[], operations: Operation[]) => void) | undefined;
}) => JSX.Element;
/**
 * Get the current `Editor` class that the component lives under.
 */
export declare const useSlate: () => Editor;
//# sourceMappingURL=use-slate.d.ts.map